@@ -12,17 +12,17 @@ test_that("Corpus comparison works!", {
            'In the meantime, Mark Rutte is still simply Rutte')
   tc_y = create_tcorpus(text_y, split_sentences = T)
 
-  comp = tc$compare_corpus(tc_y, 'token')
+  comp = compare_corpus(tc, tc_y, 'token')
   expect_equal(round(sum(comp$chi2),3), 2.215)
   #graphics::plot(comp)
   #graphics::plot(comp, mode = 'both')
 
-  comp = tc$compare_subset('token', query_x = 'rutte')
+  comp = compare_subset(tc, 'token', query_x = 'rutte')
   expect_equal(round(sum(comp$chi2),3), 5.646)
   #graphics::plot(comp)
 
-  comp = tc$compare_subset('token', query_x = 'rutte', what='docfreq', smooth=F)
+  comp = compare_subset(tc, 'token', query_x = 'rutte', what='docfreq', smooth=F)
   expect_equal(round(sum(comp$chi2),3), 4.809)
 
-  comp = tc$compare_corpus(tc_y, 'token', what = 'cooccurrence', smooth = F)
+  comp = compare_corpus(tc, tc_y, 'token', what = 'cooccurrence', smooth = F)
 })
